CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  
VAR
  long TicksPerSecond 'these variables are used to calculate always the correct values
  long WaitTime_mSecs 'even if ClkFreq should change
  long WaitTimeTicks

  long CntSnapShot  'contains SnapShot of systemtimer "cnt"
  long elapsedTime


OBJ
  debug : "FullDuplexSerial"


PUB Main
  'the FIRST PUB-Method inside a spinfile is ALWAYS the startpoint where the program starts to run    
  debug.start(31, 30, 0, 115200)

  'calculate values 
  TicksPerSecond := ClkFreq
  WaitTime_mSecs := 2000 'variable WaitTime_mSecs contains waiting-period in milliseconds
  WaitTimeTicks  := WaitTime_mSecs * (TicksPerSecond / 1000)   
  TaggedVar(string("WaitTimeTicks  := WaitTime_mSecs * (TicksPerSecond / 1000)="),WaitTimeTicks)
  
  CntSnapShot := cnt 'make inital snapshot of systemcounter 
  repeat
    'do whatever you like inside this loop. It is looping at highest possible speed
    '
    'the method elapsed_time_msec calculates the time that has gone since
    'variable CntSnapShot has set to the (former) value of systemcounter
    elapsedTime := elapsed_time_msec(CntSnapShot) 

    if elapsedTime => WaitTime_mSecs  'if waiting-period is over execute code below 
    '
      'first of all set Snapshot-value exactly one waiting-period ahead
      CntSnapShot := CntSnapShot + WaitTimeTicks 

      TaggedVar(string("Waiting period is over. elapsedTime millisecs="),elapsedTime)
      debug.tx(13)

      
PUB elapsed_time_msec(p_TimeVar) | RightNow, ClockTicks

  RightNow := cnt

  if RightNow < p_TimeVar                          
    ClockTicks := ||($FFFFFFFF - RightNow + p_TimeVar)  
  else
    ClockTicks := ||(RightNow - p_TimeVar)

  result := ClockTicks / (clkfreq / 1_000) 'calculate milliseconds
  ' SPIN treats longs as SIGNED longs meaning bit 32 represents the sign "+-"
  ' the systemcounter thinks of the 32bits as UNsigned meaning bit 32 is
  ' just another bit of the number and NOT a sign "+-" 
  ' if one or both values are negative it could happen
  ' that the result is negative too
  ' the command "||" calculates the absolute value of the SIGNED longs

  'if the systemcounter has wrapped around since snapshot of
  'systemcounter (value of parameter p_TimeVar),
  'this method calculates the timedifference in the right way
  
  'wrap-around example with easy numbers: counter maxvalue 1000
  'p_TimeVar containing value 980
  'time goes on counter wraps around from 1000 to 0
  'time goes on 
  'RightNow containing 300. This means 20 ticks until maximum 1000 and 300 ticks
  'since wrapping from 1000 to 0 in summary 320 ticks of time has gone
  'this is calculated by MaxCounter - p_TimeVar + RightNow
  'in numbers              1000     -   980     +   300     = 320
  'the real systemcounter has 32bits max value 2^32 -1 = 4294967295
  'hexadecimal $FFFFFFFF
 
'end of PUB elapsed_time_msec(p_TimeVar) | RightNow, ClockTicks


PUB TaggedVar(p_Tag,p_Var)
'That's a method for easily debugging code
'example of use
'long MyVar
'TaggedVar(string("MyVar has value:"),MyVar)
  debug.str(p_Tag)
  debug.dec(p_Var)
  debug.Tx(13)
